#include <clooptools.h>
#include <fstream.h>
#include "MSSMparam.h"
#include "SUSYnum.h"

#include "qag.h"
#include "mcdefs.h"	
	
	
#define BORN	     1
#define VIRTUAL      2
#define REAL	     4

#define LIGHTLEPT    1
#define HEAVYLEPT    2
#define LIGHTQURK    4
#define HEAVYQURK    8
#define NOFERMION   16
#define QED	    32
#define QED1	    64
#define QED2	   128
#define QED3	   256

double born(double s);
double dborn(double s, double t);

double csvirtl2(double s, double t);
double csvirtl3(double s, double t);
double csvirtq2(double s, double t);
double csvirtq3(double s, double t);
double csvirtnof1(double s, double t);
double csvirtnof2(double s, double t);
double csvirtnof3(double s, double t);
double csvirtqed(double s, double t);
double csvirtqed1(double s, double t);
double csvirtqed2(double s, double t);
double csvirtqed3(double s, double t);

double csisrfac(double s, double sqrts, double Eg);
double dcsisrfac(double s, double CT, double sqrts, double Eg);

double s, sqrts;
unsigned cls, type, numc;

RESULT_TYPE intb(Double t)
{
  return dborn(s, t);
}

RESULT_TYPE intf(Double t)
{
  double res = 0.0;
  if((cls & LIGHTLEPT) != 0)
    res += csvirtl2(s, t);
  if((cls & HEAVYLEPT) != 0)
    res += csvirtl3(s, t);
  if((cls & LIGHTQURK) != 0)
    res += csvirtq2(s, t);
  if((cls & HEAVYQURK) != 0)
    res += csvirtq3(s, t);
  if((cls & NOFERMION) != 0)
    res += csvirtnof1(s, t) + csvirtnof2(s, t) + csvirtnof3(s, t);
  if((cls & QED) != 0)
    res += csvirtqed(s, t);
  if((cls & QED1) != 0)
    res += csvirtqed1(s, t);
  if((cls & QED2) != 0)
    res += csvirtqed2(s, t);
  if((cls & QED3) != 0)
    res += csvirtqed3(s, t);
  return res;
}


double egsav, cost1, cost2;
QAGCLASS dcint(FIX_RELATIVE_ERROR,0.00001);
#define CTEPS  0.001

RESULT_TYPE dcollfac(Double ct)
{
  return dcsisrfac(s, ct, sqrts, egsav);
}

RESULT_TYPE dcollfac1(Double eg)
{
  egsav = eg;
  return dcint.integrate(dcollfac, cost1, cost2);
}

RESULT_TYPE dcollfac2(Double eg)
{
  return dcsisrfac(s, cost1, sqrts, eg);
}


int main(int argc, char *argv[])
{
  double ctmin, ctmax, tmin, tmax, beta;
  mocres res, res2;
//  QAGCLASS gkint(FIX_ABSOLUTE_ERROR,0.00001);
  QAGCLASS gkint(FIX_RELATIVE_ERROR,0.00001);
	  
  if(argc >= 1)
    initppars(argv[1]);
  else
  {
    cerr << "No input file specified!" << endl;
    exit(1);
  }
  setsusypars(*argv);
  softparcorr();
  setsusypars(*argv);
  ffini();
  
  s = pars[0];
  sqrts = sqrt(s);
  ctmin = pars[98]; ctmax = pars[99];
  cls = (unsigned)(rint(pars[97]));
  type = (unsigned)(rint(pars[96]));
  numc = (unsigned)(rint(pars[95]));
  
  beta = sqrt(1-4*msmr*msmr/s);
  tmin = msmr*msmr - s/2 * (1- beta*ctmin);
  tmax = msmr*msmr - s/2 * (1- beta*ctmax);

  res.avgi = res.sd = 0.0;
  if((type & BORN) != 0)
  {
    if(ctmin == ctmax)
      res.avgi += TOFB * beta*s/2. * dborn(s,tmin);
    else
      res.avgi += TOFB * gkint.integrate(intb, tmin, tmax);
  }
  if((type & VIRTUAL) != 0)
  {
    if(ctmin == ctmax)
      res.avgi += TOFB / (16*M_PI*s*s) * beta*s/2. * intf(tmin);
    else
      res.avgi += TOFB / (16*M_PI*s*s) * gkint.integrate(intf, tmin, tmax);
  }
  if((type & REAL) != 0)
  {
    if(ctmin == ctmax)
    {
      ctmax = ctmin + CTEPS;
      if(ctmax > 1.0)
      {
	ctmax = 1.0;  ctmin = 1.0 - CTEPS;
      }
      randomize();
      printtime();
      res2 = evalMCpoint(ISR+INTERF+FAC, s, ctmin, ctmax, numc);
      res.avgi += res2.avgi/CTEPS;
      res.sd += res2.sd/CTEPS;
      cost1 = ctmin;
      res.avgi += 2*TOFB *	// factor 2 for two collinearity
                                //  regions near imcoming e+ & e-
	      gkint.integrate(dcollfac2, PHOTENCUT, sqrt(s)/2*(1.-4*msmr*msmr/s));
      res2 = evalMCpoint(FSR, s, ctmin, ctmax, numc);
      res.avgi += res2.avgi/CTEPS;
      res.sd += res2.sd/CTEPS;
      cls = QED;
      res.avgi += TOFB * beta*s/2. * intf(tmin);
    }
    else
    {
      randomize();
      printtime();
      res2 = evalMCpoint(ISR+INTERF+FAC, s, ctmin, ctmax, numc);
      res.avgi += res2.avgi;
      res.sd += res2.sd;
      cost1 = ctmin; cost2 = ctmax;
      res.avgi += 2*TOFB *	// factor 2 for two collinearity
                                //  regions near imcoming e+ & e-
	      gkint.integrate(dcollfac1, PHOTENCUT, sqrt(s)/2*(1.-4*msmr*msmr/s));
      res2 = evalMCpoint(FSR, s, ctmin, ctmax, numc);
      res.avgi += res2.avgi;
      res.sd += res2.sd;
      cls = QED;
      res.avgi += TOFB * gkint.integrate(intf, tmin, tmax);
    }
  }

  ofstream of("tint.out", ios::out );
  of.precision(8);
  of << res.avgi << " " << res.sd << endl;
  of.close();

  ffexi();
  return 0;
}
